library ieee;
use ieee.std_logic_1164.all;
use work.pack_2.all;

entity uut is
  port (x_fghij : in  X01_vector(20 downto 13);
        z_abcde : out X01_vector(20 downto 13));
end uut;

architecture structure of uut is
  component sn7400 -- 2er-nand
    port (x : in  X01_vector (1 to 2);
          y : out X01);
  end component;
  component sn7404 is -- not
    port (x : in X01; y : out X01);
  end component;
  component sn74151 is -- 8-zu-1-Multiplexer
    port (e   : in  X01;
          s   : in  X01_vector (2 downto 0);
          d   : in  X01_vector (0 to 7); 
          y,w : out X01);
  end component;
  component sn74153 is -- 4-zu-1-Multiplexer
    port (e1,e2  : in  X01;
          s      : in  X01_vector (1 downto 0);
          d1,d2  : in  X01_vector (0 to 3);
          y1,y2  : out X01);
  end component;

  signal nc, nd, zero, one : X01; -- not a, not b, not c, not
  signal s_nand : X01_vector (1 to 4); -- zwischenergebnisse der nands
  signal s_not : X01_vector (1 to 2); -- zwischenergebnisse der not bei nand

  alias a : x01 is x_fghij(20);
  alias b : x01 is x_fghij(19);
  alias c : x01 is x_fghij(18);
  alias d : x01 is x_fghij(17);
  alias y1: x01 is z_abcde(20); -- 8-zu-1-mux ergebnis
  alias y2: x01 is z_abcde(19); -- 4-zu-1-mux ergebnis
begin
  -- todo
  -- block1-not: 1, c, d => 0, not c, not d
  one <= '1';
  b11: sn7404 port map (x=>one, y=>zero);
  b12: sn7404 port map (x=>c, y=>nc);
  b13: sn7404 port map (x=>d, y=>nd);
  b14: sn7404 port map (x=>s_nand(1), y=>s_not(1));
  b15: sn7404 port map (x=>s_nand(2), y=>s_not(2));
  
  -- block1-nand
  b16: sn7400 port map (x(1)=>c, x(2)=>nd, y=>s_nand(1));
  b17: sn7400 port map (x(1)=>nc, x(2)=>nd, y=>s_nand(2));
  b18: sn7400 port map (x(1)=>c, x(2)=>d, y=>s_nand(3));
  b19: sn7400 port map (x(1)=>d, x(2)=>nc, y=>s_nand(4));
  
  -- block 2: 8-zu-1-Multiplexer
  b2: sn74151 port map (e=>zero, s(2)=>a, s(1)=>b, s(0)=>c,
                        d(0)=>zero, d(1)=>nd, d(2)=>nd, d(3)=>zero, 
                        d(4)=>one, d(5)=>nd, d(6)=>nd, d(7)=>one, y=>y1);
  
  -- block 3: 4-zu-1-Multiplexer
  b3: sn74153 port map (e1=>zero, s(1)=>a, s(0)=>b,
                        d1(0)=>s_not(1), d1(1)=>s_not(2),
                        d1(2)=>s_nand(3), d1(3)=>s_nand(4), y1=>y2,
                        e2=>one, d2=>"0000");
end structure;
